@@ -14,8 +14,8 @@ PBrefdist.lm <- function(largeModel, smallModel, nsim=1000, seed=NULL, cl=NULL,
   ##cat(".....PBrefdist.lm\n")
   t0 <- proc.time()
   .is.cluster <- !is.null(cl) && inherits(cl, "cluster")
-    
-  fun <- function(ll, ss, nsim, seed=seed){    
+
+  fun <- function(ll, ss, nsim, seed=seed){
     simdata <- simulate(ss, nsim, seed=seed)
     ee  <- new.env()
     ee$simdata <- simdata
@@ -25,7 +25,7 @@ PBrefdist.lm <- function(largeModel, smallModel, nsim=1000, seed=NULL, cl=NULL,
 
     ff.l <- update.formula(formula(ll),simdata[,ii]~.)
     ff.s <- update.formula(formula(ss),simdata[,ii]~.)
-    environment(ff.l) <- environment(ff.s) <- ee 
+    environment(ff.l) <- environment(ff.s) <- ee
     cl.l$formula <- ff.l
     cl.s$formula <- ff.s
     cl.l$data <- ss$data
@@ -35,9 +35,9 @@ PBrefdist.lm <- function(largeModel, smallModel, nsim=1000, seed=NULL, cl=NULL,
     for (ii in 1:nsim){
       ref[ii] <- 2*(logLik(eval(cl.l))-logLik(eval(cl.s)))
     }
-    ref    
+    ref
   }
-  
+
   if (!.is.cluster){
     ref <- fun(largeModel, smallModel, nsim, seed=seed)
   } else {
@@ -55,81 +55,57 @@ PBrefdist.lm <- function(largeModel, smallModel, nsim=1000, seed=NULL, cl=NULL,
   if (details>0)
     cat(sprintf("Reference distribution with %i samples; computing time: %5.2f secs. \n",
                 length(ref), ctime))
-  
+
   ref
 }
 
 
-.getref <- function(lg, sm, nsim=20, seed=NULL){
-  simdata <- as.matrix(simulate(sm, nsim=nsim, seed=seed))
-  ref     <- rep(NA, nsim)
-  for (kk in seq_len(nsim)){
-    yyy   <- simdata[,kk]
-    sm2 <- refit(sm, newresp=yyy)
-    lg2 <- refit(lg, newresp=yyy)
-    ttt   <- 2*(logLik(lg2, REML=FALSE) - logLik(sm2, REML=FALSE))
-    ref[kk]  <- ttt
-  }
-  as.numeric(ref)
+.get_ref <- function(lg, sm, nsim=20, seed=NULL){
+    simdata <- simulate(sm, use.u=FALSE, nsim=nsim, seed=seed)
+    sm.logL <- unlist(lapply(simdata, function(dd) logLik( refit(sm, newresp=dd), REML=FALSE)))
+    lg.logL <- unlist(lapply(simdata, function(dd) logLik( refit(lg, newresp=dd), REML=FALSE)))
+    ref <- unname(2*(lg.logL - sm.logL))
+    ref
 }
 
-PBrefdist.lmerMod <- PBrefdist.mer <- function(largeModel, smallModel, nsim=1000, seed=NULL, cl=NULL,details=0){
+.merMod_ref <- function(lg, sm, nsim=20, seed=NULL){
+  simdata <- simulate(sm, nsim=nsim, seed=seed)
+  unname(unlist(lapply(simdata, function(yyy){
+    sm2     <- refit(sm, newresp=yyy)
+    lg2     <- refit(lg, newresp=yyy)
+    2*(logLik( lg2, REML=FALSE ) - logLik( sm2, REML=FALSE ))
+  })))
+}
+
+PBrefdist.mer <- PBrefdist.merMod <- function(largeModel, smallModel, nsim=1000, seed=NULL, cl=NULL, details=0){
 
   t0 <- proc.time()
-
   if (getME(smallModel, "is_REML"))
-    smallModel <- update(smallModel,REML=FALSE)  
+    smallModel <- update( smallModel, REML=FALSE )
   if (getME(largeModel, "is_REML"))
-    largeModel <- update(largeModel,REML=FALSE)
-  
+    largeModel <- update( largeModel, REML=FALSE )
+
   .is.cluster <- !is.null(cl) && inherits(cl, "cluster")
-  
   if (!.is.cluster){
-    ref <- .getref(largeModel, smallModel, nsim=nsim, seed=seed)
-  } else {      
-    nsim.cl <- unlist(lapply(splitIndices(nsim, length(cl)), length))
-    clusterCall(cl, function() {library(lme4)})
+    ref <- .merMod_ref(largeModel, smallModel, nsim=nsim, seed=seed)
+  } else {
+    nsim.cl <- nsim %/% length(cl)
     clusterSetRNGStream(cl)
-    ref <- unlist(parallel::clusterMap(cl, .getref, c(largeModel), c(smallModel), nsim.cl))
+    ref <- unlist( clusterCall(cl, fun=.merMod_ref, largeModel, smallModel, nsim=nsim.cl) )
   }
-  
+
   ctime <- (proc.time()-t0)[3]
   attr(ref,"ctime")   <- ctime
   attr(ref,"samples") <- c(nsim=nsim, npos=sum(ref>0))
   if (details>0)
-    cat(sprintf("Reference distribution with %i samples; computing time: %5.2f secs. \n",
+    cat(sprintf("Reference distribution with %5i samples; computing time: %5.2f secs. \n",
                 length(ref), ctime))
-  
+
   ref
 }
 
 
 
-    ##ref <- foo(largeModel, smallModel, cl, nsim)
-    ##print(ref)
 
 
-    ##clusterCall(cl, function() {library(lme4)})
-    ##print(clusterCall(cl, function() packageVersion("lme4")))
-    ##ref <- unlist(clusterApply(cl, nsim22, function(ii){fun(largeModel, smallModel, nsim=ii)}))
-    ##clusterCall(cl, function() detach(package:lme4, unload=TRUE))
 
-
-##     ## is lme4 loaded:
-##     xx <- grep("package:lme4", clusterCall(cl, search))
-##     print(xx)
-##     ## if yes, then detach
-##     if (length(xx)){
-##       print(unlist(clusterCall(cl, function() packageVersion("lme4")))); cat("\n")
-##       cat("detaching lme4 on clusters\n")
-##       clusterCall(cl, function() detach(package:lme4, unload=TRUE))
-##     }
-
-##     Sys.sleep(2)
-
-##     cat("load lme4 (the relevant version)\n")
-##     clusterCall(cl, function(lll) {library(lme4, lib.loc=lll)}, lib.loc)
-
-## Sys.sleep(2)
-##     cat("checking versions:\n")
-##     print(unlist(clusterCall(cl, function() packageVersion("lme4")))); cat("\n")
